C File <d0wdd0.F> containing the D0 function for finite width
C This file is part of aITALC
C Copyright (C) 2004 Alejandro Lorca <alejandro.lorca@desy.de>
C-----------------------------------------------------------------------
      double complex function D0wdd0(
     &     P1Sin,P2Sin,P3Sin,P4Sin,P1P2Sin,P2P3Sin, !external momenta
     &     cM1Sin,                 !The massless particle
     &     cM2Sin,
     &     cM3Sin,                !here it comes the complex mass
     &     cM4Sin)
C-----------------------------------------------------------------------
      implicit none
      integer index
      double precision P1Sin,P2Sin,P3Sin,P4Sin,P1P2Sin,P2P3Sin
      double complex cM1Sin,cM2Sin,cM3Sin,cM4Sin
      double precision P1S,P2S,P3S,P4S,P1P2S,P2P3S
      double precision M1S,M2S,M4S
      double complex cM3S

      double precision IG3,IG2

      integer irho,isig
      double complex m0,x2,x3
      double precision m1,m4,m2,m3,s,t,xs,lambda
      double complex lambdapart,part1,part2,part3
      double precision dcondition

      double complex CDK,CDLOGIE,DILOGIE,CDDILOG,XSPENZ
      double precision DK
#include <variablelist.hf>
C-----------------------------------------------------------------------
C Now we have to apply symmetry properties to place the photon and the
C complex mass in the M1S and cM3S.

C Since we are interested on AZ boxes, they will be separeted by one
C element. This means it suffices for us to use cyclic properties

      if ((DBLE(cM1Sin).EQ.0d0).AND.(IMAG(cM3Sin).NE.0d0)) then
C do nothing
        M1S=DBLE(cM1Sin)
        M2S=DBLE(cM2Sin)
        cM3S=cM3Sin
        M4S=DBLE(cM4Sin)

        P1P2S=P1P2Sin
        P2P3S=P2P3Sin

        P1S=P1Sin
        P2S=P2Sin
        P3S=P3Sin
        P4S=P4Sin
      elseif (DBLE(cM2Sin).EQ.0d0.AND.(IMAG(cM4Sin).NE.0d0)) then
C everything one step down
        M1S=DBLE(cM2Sin)
        M2S=DBLE(cM3Sin)
        cM3S=cM4Sin
        M4S=DBLE(cM1Sin)

        P1P2S=P2P3Sin
        P2P3S=P1P2Sin

        P1S=P2Sin
        P2S=P3Sin
        P3S=P4Sin
        P4S=P1Sin
      elseif (DBLE(cM3Sin).EQ.0d0.AND.(IMAG(cM1Sin).NE.0d0)) then
C everything two steps down
        M1S=DBLE(cM3Sin)
        M2S=DBLE(cM4Sin)
        cM3S=cM1Sin
        M4S=DBLE(cM2Sin)

        P1P2S=P1P2Sin
        P2P3S=P2P3Sin

        P1S=P3Sin
        P2S=P4Sin
        P3S=P1Sin
        P4S=P2Sin
      elseif (DBLE(cM4Sin).EQ.0d0.AND.(IMAG(cM2Sin).NE.0d0)) then
C everything three steps down
        M1S=DBLE(cM4Sin)
        M2S=DBLE(cM1Sin)
        cM3S=cM2Sin
        M4S=DBLE(cM3Sin)

        P1P2S=P2P3Sin
        P2P3S=P1P2Sin

        P1S=P4Sin
        P2S=P1Sin
        P3S=P2Sin
        P4S=P3Sin
      else
        print *,'d0wdd0: Something on the argument wrong!'
      endif

C-----------------------------------------------------------------------
      m0=SQRT(cM3S)
      m1=DSQRT(M2S)
      m2=DSQRT(P2S)
      m3=DSQRT(P3S)
      m4=DSQRT(M4S)
C We cannot assign it, lambda is our regulator!!!
      lambda=DSQRT(MMASoft)

      s=P2P3S
      t=P1P2S

      xs=-DK(s,m1,m4)
      x2=-CDK(m2**2,m1,m0)
      x3=-CDK(m3**2,m4,m0)
      dcondition = DABS(1-(m1-m4)**2/s)
C-----------------------------------------------------------------------
C The calculation itself has two cases:
      if (dcondition.LT.1D-14) then
        print *,'D0w: case of s=(m1-m4)^2, please provide formula!'
        stop
      else

        lambdapart=LOG(m0*lambda/(m0**2-t))

        part1=2*CDLOGIE(xs,+1) * (
     &       +CDLOGIE(1-xs**2,-1)
     &       -lambdapart
     &       )
*        print *,'part1=',part1
        
        part2=Pi**2/2
     &       +DILOGIE(xs**2,+1)
     &       +(LOG(x2))**2
     &       +(LOG(x3))**2
*        print *,'part2=',part2
        

        part3=DCMPLX(0d0,0d0)
        do irho=-1,1,2
          do isig=-1,1,2
*        print *,'Arguments=',xs,x2**(irho)*x3**(isig)
            part3=part3
     &           +CDDILOG(xs*x2**(irho)*x3**(isig))
     &           +(
     &           +CDLOGIE(xs,+1)
     &           +LOG(x2**(irho))
     &           +LOG(x3**(isig))
     &           )*LOG(1-xs*x2**(irho)*x3**(isig))
          enddo
        enddo
*        print *,'part3=',part3
*        print *,'prod =',xs/(m1*m4*(t-m0**2)*(1-xs**2))
*        stop
        D0wdd0=xs/(m1*m4*(t-m0**2)*(1-xs**2))*(
     &       +part1
     &       +part2
     &       -part3
     &       )

      endif
C-----------------------------------------------------------------------
      return
      end

C***********************************************************************
C Implementation of K function as BeDe'90, eq (2.7)

      double precision function DK(x,m,mp)
C-----------------------------------------------------------------------
      implicit none
      double precision x,m,mp
      double precision difference,condition,squareroot,dterm
C-----------------------------------------------------------------------
      difference=x-(m-mp)**2
      condition=DABS(difference/x)
      dterm=4*m*mp/difference
C-----------------------------------------------------------------------
      if ((difference.LT.4*m*mp).AND.(difference.GT.0d0)) then
        print *,'DK: Kinematical threshold not passed!'
        stop
      elseif (condition.LT.1D-15) then
        DK=-1d0
      else
        if (DABS(dterm).LT.1D-6) then
c          print *,'Computed with expansion O(dterm^2)'
          DK=dterm/4*(1
     &         +dterm/2
     &         +dterm**2/4
     &         )
        else
c          print *,'Computed with DSQRT'
          squareroot=DSQRT(1-4*m*mp/difference)
          DK=(1-squareroot)/(1+squareroot)
        endif
      endif
C-----------------------------------------------------------------------
      return
      end

C***********************************************************************
C Implementation of K function as BeDe'90, eq (2.7) for complex mass

      double complex function CDK(z,m,cm)
C-----------------------------------------------------------------------
      implicit none
      double precision z,m,condition
      double complex cm
      double complex cdifference,csquareroot,cterm
C-----------------------------------------------------------------------
      cdifference=z-(m-cm)**2
      condition=ABS(cdifference/z)
      cterm=4*m*cm/cdifference
C-----------------------------------------------------------------------
      if (condition.LT.1D-15) then
        print *,'To small to be true, check z-(m-cm)**2=0'
        CDK=DCMPLX(-1d0,0d0)
      else
        if (ABS(cterm).LT.1D-6) then
c          print *,'Computed with expansion O(cterm^2)'
          CDK=DCMPLX(DBLE(cterm)-IMAG(cterm)**2,-(IMAG(cterm)**2)/2)
     &         /2
c          print *,'CDK=',CDK
        else
c          print *,'Computed with SQRT'
          csquareroot=SQRT(1-4*m*cm/cdifference)
          CDK=(1-csquareroot)/(1+csquareroot)
        endif
      endif
C-----------------------------------------------------------------------
      return
      end

CC $Id: d0wdd0.F,v 1.4 2004/10/14 11:27:27 alorca Exp $
